//+---------------------------------------------------------------------+
//|                                              Accelerator_Signal.mq5 |
//|                                       Copyright  2007, Asystem2000 | 
//|                                                                     | 
//+---------------------------------------------------------------------+ 
//|       SmoothAlgorithms.mqh    |
//|   (): __\\MQL5\Include        |
//+---------------------------------------------------------------------+
#property copyright "Copyright  2007, Asystem2000"
#property link "" 
//----   
#property version   "1.00"
//----     
#property indicator_separate_window 
//----    4
#property indicator_buffers 4 
//----     
#property indicator_plots   2
//+----------------------------------------------+
//|     1            |
//+----------------------------------------------+
//----      
#property indicator_type1   DRAW_FILLING
//----     
#property indicator_color1  clrDeepSkyBlue,clrMagenta
//----   
#property indicator_label1  "Accelerator Signal"
//+----------------------------------------------+
//|     2            |
//+----------------------------------------------+
//----      
#property indicator_type2 DRAW_COLOR_HISTOGRAM
//----      
#property indicator_color2 clrDeepPink,clrOrange,clrGray,clrLimeGreen,clrTeal
//----   - 
#property indicator_style2 STYLE_SOLID
//----     2
#property indicator_width2 2
//----   
#property indicator_label2  "Accelerator"
//+-----------------------------------+
//|          |
//+-----------------------------------+
#include <SmoothAlgorithms.mqh> 
//+-----------------------------------+
//----    CXMA   SmoothAlgorithms.mqh
CXMA XMA1,XMA2,XMA3,XMA4;
//+-----------------------------------+
//|             |
//+-----------------------------------+
enum Applied_price_      //  
  {
   PRICE_CLOSE_ = 1,     // Close
   PRICE_OPEN_,          // Open
   PRICE_HIGH_,          // High
   PRICE_LOW_,           // Low
   PRICE_MEDIAN_,        // Median Price (HL/2)
   PRICE_TYPICAL_,       // Typical Price (HLC/3)
   PRICE_WEIGHTED_,      // Weighted Close (HLCC/4)
   PRICE_SIMPL_,         // Simple Price (OC/2)
   PRICE_QUARTER_,       // Quarted Price (HLOC/4) 
   PRICE_TRENDFOLLOW0_,  // TrendFollow_1 Price 
   PRICE_TRENDFOLLOW1_,  // TrendFollow_2 Price 
   PRICE_DEMARK_         // Demark Price 
  };
//+-----------------------------------+
//|         |
//+-----------------------------------+
input Smooth_Method XMA_Method=MODE_SMA;        //   Accelerator 
input int Fast_XMA=5;                           //   
input int Slow_XMA = 34;                        //   
input int XPhase = 100;                         //   
//---- XPhase:  JJMA    -100 ... +100,     ;
//---- XPhase:  VIDIA   CMO,  AMA    
input Smooth_Method Sm_Method=MODE_SMA;         //    
input int Sm_XMA=5;                             //    
input int Sm_Phase=100;                         //   
input Smooth_Method Signal_Method=MODE_SMA;     //    
input int Signal_XMA=5;                         //    
input int Signal_Phase=100;                     //   
input Applied_price_ AppliedPrice=PRICE_CLOSE;  //  
//+-----------------------------------+
//----      
int min_rates_total,min_rates_1,min_rates_2;
//----   ,   
//----      
double IndBuffer[],ColorIndBuffer[],UpSignBuffer[],DnSignBuffer[];
//+------------------------------------------------------------------+    
//| Accelerator indicator initialization function                    | 
//+------------------------------------------------------------------+  
void OnInit()
  {
//----     
   min_rates_1=MathMax(XMA1.GetStartBars(XMA_Method,Fast_XMA,XPhase),XMA1.GetStartBars(XMA_Method,Slow_XMA,XPhase));
   min_rates_2=min_rates_1+XMA1.GetStartBars(Sm_Method,Sm_XMA,Sm_Phase);
   min_rates_total=min_rates_2+XMA1.GetStartBars(Signal_Method,Signal_XMA,Signal_Phase)+2;
//----    SignBuffer   
   SetIndexBuffer(0,UpSignBuffer,INDICATOR_DATA);
//----     ,     
   SetIndexBuffer(1,DnSignBuffer,INDICATOR_DATA);   
//----    XMACDBuffer   
   SetIndexBuffer(2,IndBuffer,INDICATOR_DATA);  
//----          
   SetIndexBuffer(3,ColorIndBuffer,INDICATOR_COLOR_INDEX);
//----      
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,min_rates_total);
//----   ,      
   PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,EMPTY_VALUE);
//----       
   XMA1.XMALengthCheck("Fast_XMA", Fast_XMA);
   XMA1.XMALengthCheck("Slow_XMA", Slow_XMA);
   XMA1.XMALengthCheck("Sm_XMA", Sm_XMA);
   XMA1.XMALengthCheck("Signal_XMA", Signal_XMA);
//----       
   XMA1.XMAPhaseCheck("XPhase", XPhase, XMA_Method);
   XMA1.XMAPhaseCheck("Sm_Phase", Sm_Phase, Sm_Method);
   XMA1.XMAPhaseCheck("Signal_Phase", Signal_Phase, Signal_Method);
//----      
   string shortname;
   string Smooth1=XMA1.GetString_MA_Method(XMA_Method);
   string Smooth2=XMA1.GetString_MA_Method(Signal_Method);
   StringConcatenate(shortname,"Accelerator( ",Fast_XMA,", ",Slow_XMA,", ",Sm_XMA,", ",Signal_XMA,", ",Smooth1,", ",Smooth2," )");
//---           
   IndicatorSetString(INDICATOR_SHORTNAME,shortname);
//---     
   IndicatorSetInteger(INDICATOR_DIGITS,0);
//----  
  }
//+------------------------------------------------------------------+  
//| Accelerator iteration function                                   | 
//+------------------------------------------------------------------+  
int OnCalculate(const int rates_total,    //       
                const int prev_calculated,//       
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//----       
   if(rates_total<min_rates_total) return(0);

//----   
   int first1,first2,bar;
//----       
   double price_,fast_xma,slow_xma,xmacd,sign_xma,accel,xaccel;

//----     OnCalculate()
   if(prev_calculated>rates_total || prev_calculated<=0)//      
     {
      first1=0; //        
      first2=min_rates_1+1; //        
     }
   else //      
     {
      first1=prev_calculated-1;
      first2=first1;
     }

//----    
   for(bar=first1; bar<rates_total; bar++)
     {
      price_=PriceSeries(AppliedPrice,bar,open,low,high,close);;
      fast_xma=XMA1.XMASeries(0,prev_calculated,rates_total,XMA_Method,XPhase,Fast_XMA,price_,bar,false);
      slow_xma=XMA2.XMASeries(0,prev_calculated,rates_total,XMA_Method,XPhase,Slow_XMA,price_,bar,false);
      xmacd=(fast_xma-slow_xma)/_Point;
      sign_xma=XMA3.XMASeries(min_rates_1,prev_calculated,rates_total,Sm_Method,Sm_Phase,Sm_XMA,xmacd,bar,false);
      accel=xmacd-sign_xma;
      xaccel=XMA4.XMASeries(min_rates_2,prev_calculated,rates_total,Signal_Method,Signal_Phase,Signal_XMA,accel,bar,false);
      //----          
      IndBuffer[bar]=accel;
      UpSignBuffer[bar]=accel;
      DnSignBuffer[bar]=xaccel;
     }
//----     XMACD
   for(bar=first2; bar<rates_total; bar++)
     {
      int clr=2;

      if(IndBuffer[bar]>0)
        {
         if(IndBuffer[bar]>IndBuffer[bar-1]) clr=4;
         if(IndBuffer[bar]<IndBuffer[bar-1]) clr=3;
        }

      if(IndBuffer[bar]<0)
        {
         if(IndBuffer[bar]<IndBuffer[bar-1]) clr=0;
         if(IndBuffer[bar]>IndBuffer[bar-1]) clr=1;
        }

      ColorIndBuffer[bar]=clr;
     }
//----     
   return(rates_total);
  }
//+------------------------------------------------------------------+
